@@ -5,9 +5,15 @@
 #include <sstream>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <unistd.h>
 #include <unordered_set>
 
+#ifdef _WIN32
+# define _isatty isatty
+# define STDERR_FILENO (_fileno(stderr))
+#else
+# include <unistd.h>
+#endif
+
 namespace mold::elf {
 
 static const char helpmsg[] = R"(
@@ -185,8 +191,8 @@ Options:
     -z notext
     -z textoff
 
-mold: supported targets: elf32-i386 elf64-x86-64 elf64-littleaarch64
-mold: supported emulations: elf_i386 elf_x86_64 aarch64linux aarch64elf)";
+mold: supported targets: elf32-i386 elf64-x86-64 elf32-littlearm elf64-littleaarch64 elf32-littleriscv elf64-littleriscv
+mold: supported emulations: elf_i386 elf_x86_64 armelf_linux_eabi aarch64linux aarch64elf elf32lriscv elf64lriscv)";
 
 static std::vector<std::string> add_dashes(std::string name) {
   // Single-letter option
@@ -209,7 +215,7 @@ static i64 parse_hex(Context<E> &ctx, std::string opt, std::string_view value) {
   static std::regex re(R"((?:0x|0X)?([0-9a-fA-F]+))", flags);
 
   std::cmatch m;
-  if (!std::regex_match(value.begin(), value.end(), m, re))
+  if (!std::regex_match(value.data(), value.data() + value.size(), m, re))
     Fatal(ctx) << "option -" << opt << ": not a hexadecimal number";
   return std::stoul(m[1], nullptr, 16);
 }
@@ -329,8 +335,8 @@ std::vector<std::string> parse_nonpositional_args(Context<E> &ctx) {
   std::vector<std::string> remaining;
   std::string_view arg;
 
-  ctx.arg.color_diagnostics = isatty(STDERR_FILENO);
   ctx.page_size = E::page_size;
+  ctx.arg.color_diagnostics = isatty(STDERR_FILENO);
 
   bool version_shown = false;
   bool warn_shared_textrel = false;
@@ -798,7 +804,8 @@ std::vector<std::string> parse_nonpositional_args(Context<E> &ctx) {
     } else if (read_arg("opt-remarks-format")) {
       ctx.arg.plugin_opt.push_back("opt-remarks-format=" + std::string(arg));
     } else if (read_arg("opt-remarks-hotness-threshold")) {
-      ctx.arg.plugin_opt.push_back("opt-remarks-hotness-threshold=" + std::string(arg));
+      ctx.arg.plugin_opt.push_back("opt-remarks-hotness-threshold=" +
+                                   std::string(arg));
     } else if (read_arg("opt-remarks-passes")) {
       ctx.arg.plugin_opt.push_back("opt-remarks-passes=" + std::string(arg));
     } else if (read_flag("opt-remarks-with_hotness")) {
@@ -810,7 +817,8 @@ std::vector<std::string> parse_nonpositional_args(Context<E> &ctx) {
       ctx.arg.plugin_opt.push_back("O" + std::string(args[0].substr(7)));
       args = args.subspan(1);
     } else if (read_arg("lto-pseudo-probe-for-profiling")) {
-      ctx.arg.plugin_opt.push_back("pseudo-probe-for-profiling=" + std::string(arg));
+      ctx.arg.plugin_opt.push_back("pseudo-probe-for-profiling=" +
+                                   std::string(arg));
     } else if (read_arg("lto-sample-profile")) {
       ctx.arg.plugin_opt.push_back("sample-profile=" + std::string(arg));
     } else if (read_flag("save-temps")) {
@@ -822,7 +830,8 @@ std::vector<std::string> parse_nonpositional_args(Context<E> &ctx) {
     } else if (read_flag("thinlto-index-only")) {
       ctx.arg.plugin_opt.push_back("thinlto-index-only");
     } else if (read_arg("thinlto-object-suffix-replace")) {
-      ctx.arg.plugin_opt.push_back("thinlto-object-suffix-replace=" + std::string(arg));
+      ctx.arg.plugin_opt.push_back("thinlto-object-suffix-replace=" +
+                                   std::string(arg));
     } else if (read_arg("thinlto-prefix-replace")) {
       ctx.arg.plugin_opt.push_back("thinlto-prefix-replace=" + std::string(arg));
     } else if (read_arg("thinlto-cache-dir")) {
