@@ -44,6 +44,7 @@ enum PluginTag {
   LDPT_REGISTER_NEW_INPUT_HOOK,
   LDPT_GET_WRAP_SYMBOLS,
   LDPT_ADD_SYMBOLS_V2,
+  LDPT_GET_API_VERSION,
 };
 
 enum PluginApiVersion {
@@ -141,6 +142,11 @@ enum PluginLevel {
   LDPL_FATAL,
 };
 
+enum PluginLinkerAPIVersion {
+  LAPI_V0 = 0,
+  LAPI_V1,
+};
+
 typedef PluginStatus OnloadFn(PluginTagValue *tv);
 typedef PluginStatus ClaimFileHandler(const PluginInputFile *, int *);
 typedef PluginStatus AllSymbolsReadHandler();
