@@ -6,9 +6,12 @@
 #include <sstream>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <unistd.h>
 #include <unordered_set>
 
+#ifndef _WIN32
+#include <unistd.h>
+#endif
+
 namespace mold::macho {
 
 static const char helpmsg[] = R"(
@@ -63,7 +66,8 @@ Options:
                               Allocate MAXPATHLEN byte padding after load commands
   -help                       Report usage information
   -hidden-l<LIB>
-  -ignore_optimization_hints  Do not rewrite instructions as optimization
+  -ignore_optimization_hints  Do not rewrite instructions as optimization (default)
+    -enable_optimization_hints
   -install_name <NAME>
   -l<LIB>                     Search for a given library
   -lto_library <FILE>         Ignored
@@ -139,7 +143,7 @@ i64 parse_version(Context<E> &ctx, std::string_view arg) {
   static std::regex re(R"((\d+)(?:\.(\d+))?(?:\.(\d+))?)",
                        std::regex_constants::ECMAScript);
   std::cmatch m;
-  if (!std::regex_match(arg.begin(), arg.end(), m, re))
+  if (!std::regex_match(arg.data(), arg.data() + arg.size(), m, re))
     Fatal(ctx) << "malformed version number: " << arg;
 
   i64 major = (m[1].length() == 0) ? 0 : stoi(m[1]);
@@ -391,6 +395,8 @@ std::vector<std::string> parse_nonpositional_args(Context<E> &ctx) {
       remaining.push_back(std::string(arg));
     } else if (read_flag("-ignore_optimization_hints")) {
       ctx.arg.ignore_optimization_hints = true;
+    } else if (read_flag("-enable_optimization_hints")) {
+      ctx.arg.ignore_optimization_hints = false;
     } else if (read_arg("-install_name") || read_arg("-dylib_install_name")) {
       ctx.arg.install_name = arg;
     } else if (read_joined("-l")) {
