@@ -2,12 +2,15 @@
 #include "../sha.h"
 
 #include <shared_mutex>
-#include <sys/mman.h>
 #include <tbb/enumerable_thread_specific.h>
 #include <tbb/parallel_for.h>
 #include <tbb/parallel_for_each.h>
 #include <tbb/parallel_sort.h>
 
+#ifndef _WIN32
+#include <sys/mman.h>
+#endif
+
 namespace mold::macho {
 
 template <typename E>
@@ -1237,7 +1240,7 @@ void ObjcImageInfoSection<E>::copy_buf(Context<E> &ctx) {
 
 template <typename E>
 void CodeSignatureSection<E>::compute_size(Context<E> &ctx) {
-  std::string filename = filepath(ctx.arg.final_output).filename();
+  std::string filename = filepath(ctx.arg.final_output).filename().string();
   i64 filename_size = align_to(filename.size() + 1, 16);
   i64 num_blocks = align_to(this->hdr.offset, E::page_size) / E::page_size;
   this->hdr.size = sizeof(CodeSignatureHeader) + sizeof(CodeSignatureBlobIndex) +
@@ -1257,7 +1260,7 @@ void CodeSignatureSection<E>::write_signature(Context<E> &ctx) {
   u8 *buf = ctx.buf + this->hdr.offset;
   memset(buf, 0, this->hdr.size);
 
-  std::string filename = filepath(ctx.arg.final_output).filename();
+  std::string filename = filepath(ctx.arg.final_output).filename().string();
   i64 filename_size = align_to(filename.size() + 1, 16);
   i64 num_blocks = align_to(this->hdr.offset, E::page_size) / E::page_size;
 
@@ -1301,7 +1304,7 @@ void CodeSignatureSection<E>::write_signature(Context<E> &ctx) {
   auto compute_hash = [&](i64 i) {
     u8 *start = ctx.buf + i * E::page_size;
     u8 *end = ctx.buf + std::min<i64>((i + 1) * E::page_size, this->hdr.offset);
-    SHA256(start, end - start, buf + i * SHA256_SIZE);
+    sha256_hash(start, end - start, buf + i * SHA256_SIZE);
   };
 
   for (i64 i = 0; i < num_blocks; i += 1024) {
@@ -1319,7 +1322,7 @@ void CodeSignatureSection<E>::write_signature(Context<E> &ctx) {
   // entire file. We compute its value as a tree hash.
   if (ctx.arg.uuid == UUID_HASH) {
     u8 uuid[SHA256_SIZE];
-    SHA256(ctx.buf + this->hdr.offset, this->hdr.size, uuid);
+    sha256_hash(ctx.buf + this->hdr.offset, this->hdr.size, uuid);
 
     // Indicate that this is UUIDv4 as defined by RFC4122.
     uuid[6] = (uuid[6] & 0b00001111) | 0b01010000;
