@@ -18,18 +18,25 @@
 #include <sstream>
 #include <string>
 #include <string_view>
-#include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <tbb/concurrent_vector.h>
 #include <tbb/enumerable_thread_specific.h>
-#include <unistd.h>
 #include <vector>
 
-#define XXH_INLINE_ALL 1
-#include <xxhash/xxhash.h>
+#ifdef _WIN32
+#include <io.h>
+#else
+#include <sys/mman.h>
+#include <unistd.h>
+#endif
 
-#ifdef NDEBUG
+#define XXH_INLINE_ALL 1
+#include "third-party/xxhash/xxhash.h"
+
+#ifdef _MSC_VER
+#  define unreachable() __assume(0)
+#elif defined(NDEBUG)
 #  define unreachable() __builtin_unreachable()
 #else
 #  define unreachable() assert(0 && "unreachable")
@@ -74,7 +81,8 @@ inline thread_local bool opt_demangle;
 inline u8 *output_buffer_start = nullptr;
 inline u8 *output_buffer_end = nullptr;
 
-extern const std::string mold_version;
+inline std::string mold_version;
+extern std::string mold_git_hash;
 
 std::string_view errno_string();
 void cleanup();
@@ -319,7 +327,9 @@ inline u64 read_uleb(std::string_view &str) {
 }
 
 inline i64 uleb_size(u64 val) {
+#if __GNUC__
 #pragma GCC unroll 8
+#endif
   for (int i = 1; i < 9; i++)
     if (val < ((u64)1 << (7 * i)))
       return i;
@@ -753,6 +763,9 @@ public:
   MappedFile *parent = nullptr;
   MappedFile *thin_parent = nullptr;
   int fd = -1;
+#ifdef _WIN32
+  HANDLE file_handle = INVALID_HANDLE_VALUE;
+#endif
 };
 
 template <typename C>
@@ -760,7 +773,14 @@ MappedFile<C> *MappedFile<C>::open(C &ctx, std::string path) {
   if (path.starts_with('/') && !ctx.arg.chroot.empty())
     path = ctx.arg.chroot + "/" + path_clean(path);
 
-  i64 fd = ::open(path.c_str(), O_RDONLY);
+
+  i64 fd;
+#ifdef _WIN32
+    fd = ::_open(path.c_str(), O_RDONLY);
+#else
+    fd = ::open(path.c_str(), O_RDONLY);
+#endif
+
   if (fd == -1)
     return nullptr;
 
@@ -774,18 +794,32 @@ MappedFile<C> *MappedFile<C>::open(C &ctx, std::string path) {
   mf->name = path;
   mf->size = st.st_size;
 
-#ifdef __APPLE__
-  mf->mtime = (u64)st.st_mtimespec.tv_sec * 1000000000 + st.st_mtimespec.tv_nsec;
+#ifdef _WIN32
+    mf->mtime = st.st_mtime;
+#elif defined(__APPLE__)
+    mf->mtime = (u64)st.st_mtimespec.tv_sec * 1000000000 + st.st_mtimespec.tv_nsec;
 #else
-  mf->mtime = (u64)st.st_mtim.tv_sec * 1000000000 + st.st_mtim.tv_nsec;
+    mf->mtime = (u64)st.st_mtim.tv_sec * 1000000000 + st.st_mtim.tv_nsec;
 #endif
 
   if (st.st_size > 0) {
+#ifdef _WIN32
+    HANDLE handle = CreateFileMapping((HANDLE)_get_osfhandle(fd),
+                                      nullptr, PAGE_READWRITE, 0,
+                                      st.st_size, nullptr);
+    if (!handle)
+      Fatal(ctx) << path << ": CreateFileMapping failed: " << GetLastError();
+    mf->file_handle = handle;
+    mf->data = (u8 *)MapViewOfFile(handle, FILE_MAP_ALL_ACCESS, 0, 0, st.st_size);
+    if (!mf->data)
+      Fatal(ctx) << path << ": MapViewOfFile failed: " << GetLastError();
+#else
     mf->data = (u8 *)mmap(nullptr, st.st_size, PROT_READ | PROT_WRITE,
                           MAP_PRIVATE, fd, 0);
     if (mf->data == MAP_FAILED)
       Fatal(ctx) << path << ": mmap failed: " << errno_string();
-  }
+#endif
+    }
 
   close(fd);
   return mf;
@@ -813,8 +847,16 @@ MappedFile<C>::slice(C &ctx, std::string name, u64 start, u64 size) {
 
 template <typename C>
 MappedFile<C>::~MappedFile() {
-  if (size && !parent)
-    munmap(data, size);
+  if (size == 0 || parent)
+    return;
+
+#ifdef _WIN32
+  UnmapViewOfFile(data);
+  if (file_handle != INVALID_HANDLE_VALUE)
+    CloseHandle(file_handle);
+#else
+  munmap(data, size);
+#endif
 }
 
 } // namespace mold
