@@ -9,8 +9,7 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/macho/$testname
+t=out/test/macho/$MACHINE/$testname
 mkdir -p $t
 
 cat <<EOF | $CC -c -o $t/a.o -xc -
@@ -25,10 +24,10 @@ cat <<EOF | $CC -c -o $t/b.o -xc -
 int foo() { return 42; }
 EOF
 
-clang -o $t/exe1 $t/a.o -Wl,-U,_foo
+clang --ld-path=./ld64 -o $t/exe1 $t/a.o -Wl,-U,_foo
 $t/exe1 | grep -q '^5$'
 
-clang -o $t/exe2 $t/a.o $t/b.o -Wl,-U,_foo
+clang --ld-path=./ld64 -o $t/exe2 $t/a.o $t/b.o -Wl,-U,_foo
 $t/exe2 | grep -q '^42$'
 
 echo OK
